from .gasset import GAsset


def type_to_str(t):
    
    t = str(t)

    if "int" in t:
        return "int"
    elif "float" in t:
        return "float"
    elif "dict" in t:
        return "json"
    elif "bool" in t:
        return "boolean"
    else:
        return "invalid"

class Log(GAsset):
    def __init__(self, name, value, title=None, description=None, var=None, messageType="file"):
        super().__init__(name=name, title=title, description=description, var=var, messageType=messageType)
        self.set_value(value)

    def to_json(self):
        
        result = super().to_json()
        result['path'] = None
        result['value'] = self.value
        result['valueType'] = type_to_str(self.valueType)
        return result

    def set_value(self, newValue):
        self.value = newValue
        self.valueType = type(self.value)



from datetime import datetime
from ..storages.storage import Storage
from ..notifier import Notifier

class GAsset():

    storage = Storage()
    notifier = Notifier()

    def __init__(self, name, messageType, title=None, description=None, var=None, show=True) -> None:
        self.name = name
        self.messageType = messageType
        self.title = title
        self.description = description
        self.var = var
        self.created = datetime.now()
        self.modified = self.created
        self.show = show

    def to_json(self):

        result ={
            "name": self.name,
            "messageType": self.messageType,
            "title": self.title,
            "description": self.description,
            "var": self.var,
            "created": str(self.created),
            "modified": str(datetime.now()),
            "show": self.show
        }
        return result
    
    def get_path(self):
        return None
    
    def send(self):
        self.storage.save(path=self.get_path(), metadata=self.to_json())
        self.notifier.something_has_changed(self.to_json())



from .storage import Storage
import os
import glob
import json
from alidaargparser import get_asset_property
import boto3

class Minio(Storage):
    storage_type = os.path.basename(__file__).split('.py')[0]
    
    def __init__(self, storage_type=None):
        super().__init__(storage_type)

        if self.base_path is None:
            raise Exception("Base path must be set!")

        endpoint_url = get_asset_property(asset_name="go_manager", property='minIO_URL')
        aws_access_key_id = get_asset_property(asset_name="go_manager", property='minIO_ACCESS_KEY')
        aws_secret_access_key = get_asset_property(asset_name="go_manager", property='minIO_SECRET_KEY')
        
        s3 = boto3.resource('s3', 
            endpoint_url = "http://" + endpoint_url,
            aws_access_key_id = aws_access_key_id, 
            aws_secret_access_key = aws_secret_access_key)
        self.bucket = s3.Bucket(get_asset_property(asset_name="go_manager", property='minio_bucket') )


    def list_files(self):
        return glob.glob(os.path.join(self.base_path, '*.json'))
    
    def save(self, path, metadata):
    
        # Check if is a file or a log
        isFile = True
        if path is None:
            path = metadata['name']
            isFile = False

        dst = os.path.join(self.base_path, os.path.basename(os.path.normpath(path)))
        
        if isFile:
            self.bucket.upload_file(path, dst) #, ExtraArgs={'Metadata': {'a': '2'}})

        metadataDst = path + ".json"
        with open(metadataDst, "w") as write_file:
            json.dump(metadata, write_file, indent=4)
        self.bucket.upload_file(metadataDst, dst + ".json")

    # def remove(self, path):
    #     dst = os.path.join(self.base_path, os.path.basename(os.path.normpath(path)))
    #     os.remove(dst)
    #     metadataDst = dst + ".json"
    #     os.remove(metadataDst)



from .storage import Storage
import os
import glob
import shutil
import json


class Filesystem(Storage):
    storage_type = os.path.basename(__file__).split('.py')[0]
    
    def __init__(self, storage_type=None):
        super().__init__(storage_type)

        # If not specified use /resources. Helps running locally without passing too many parameters
        if self.base_path is None:
            self.base_path = "/resources/generic_outputs"

    def list_files(self):
        return glob.glob(os.path.join(self.base_path, '*.json'))
    
    def save(self, path, metadata):
        
        # Check if is a file or a log
        isFile = True
        if path is None:
            path = metadata['name']
            isFile = False

        dst = os.path.join(self.base_path, os.path.basename(os.path.normpath(path)))
        if isFile:
            os.makedirs(os.path.dirname(dst), exist_ok=True)
            shutil.copyfile(path, dst)
        metadataDst = dst + ".json"
        with open(metadataDst, "w") as write_file:
            json.dump(metadata, write_file, indent=4)

    # def remove(self, path):
    #     dst = os.path.join(self.base_path, os.path.basename(os.path.normpath(path)))
    #     os.remove(dst)
    #     metadataDst = dst + ".json"
    #     os.remove(metadataDst)


from abc import ABC, abstractmethod
from alidaargparser import get_asset_property


class Storage(ABC):

    def __new__(cls, *args, **kw):
        
        
        if 'storage_type' in kw and kw['storage_type'] is not None:
            storage_type = kw['storage_type'].lower()
        elif get_asset_property(asset_name="go_manager", property="storage_type") is not None:
            storage_type = get_asset_property(asset_name="go_manager", property="storage_type")
        else:
            storage_type = "filesystem"

        # Create a map of all subclasses based on storage_type property (present on each subclass)
        subclass_map = {subclass.storage_type: subclass for subclass in cls.__subclasses__()}


        # Select the proper subclass based on
        subclass = subclass_map[storage_type]
        instance = super(Storage, subclass).__new__(subclass)
        return instance
    
    def __init__(self, storage_type = None):
        self.base_path = get_asset_property(asset_name="go_manager", property="base_path")
        super().__init__()

    # @abstractmethod
    # def list_files(self):
    #     pass

    @abstractmethod
    def save(self, path, metadata, isFile=True):
        pass




from kafka import KafkaProducer
from bdaserviceutils import get_kafka_binder_brokers
from alidaargparser import get_asset_property
import json

class Notifier:
    def __init__(self) -> None:
        self.isActive = False

        if get_asset_property(asset_name="go_manager", property="brokers") is not None:
            self.producer = KafkaProducer(bootstrap_servers=[get_asset_property(asset_name="go_manager", property="brokers").split(",")])
            self.topic = get_asset_property(asset_name="go_manager", property="topic")
            self.isActive = True

    def something_has_changed(self, metadata):
        if self.isActive:
            message = {"message": "Generic output updated", "metatada": metadata.get()}
            self.producer.send(self.topic, json.dumps(message).encode('utf-8'))




from .gasset import GAsset
import os

class File(GAsset):
    def __init__(self, name, localPath, title=None, description=None, var=None, messageType="file"):
        super().__init__(name=name, title=title, description=description, var=var, messageType=messageType)
        self.localPath = localPath

    def set_path(self, path):
        self.localPath = path

    def get_path(self):
        return self.localPath
    
    def update_infos(self):
        self.set_extension_based_on_path()
        self.set_filename_based_on_path()
        
    def set_extension_based_on_path(self):
        self.extension = self.localPath.split(".")[-1]

    def set_filename_based_on_path(self):
        self.filename = os.path.basename(os.path.normpath(self.localPath))

    def to_json(self):
        
        self.update_infos()

        result = super().to_json()
        result['localPath'] = self.localPath 
        result['path'] = None
        result['extension'] = self.extension
        result['filename'] = self.filename

        return result

